@@ -5,7 +5,6 @@ use warnings;
 
 use parent 'PLS::Server::Request';
 
-use Cwd ();
 use Fcntl ();
 use File::Basename;
 use File::Spec;
@@ -32,6 +31,22 @@ These diagnostics currently include compilation errors and linting (using L<perl
 
 =cut
 
+my $function = IO::Async::Function->new(
+    max_workers => 1,
+    code => sub {
+        my ($uri, $unsaved, $text, $config, $root_path, $perl_exe) = @_;
+
+        $PLS::Server::State::CONFIG = $config;
+        $PLS::Server::State::ROOT_PATH = $root_path;
+        PLS::Parser::Pod->set_perl_exe($perl_exe);
+        return PLS::Server::Request::Diagnostics::PublishDiagnostics->new(uri => $uri, text => $text, unsaved => $unsaved);
+    }
+);
+
+my $loop = IO::Async::Loop->new();
+$loop->add($function);
+$function->start();
+
 sub new
 {
     my ($class, %args) = @_;
@@ -41,17 +56,18 @@ sub new
 
     return unless (ref $path eq 'URI::file');
     $path = $path->file;
-    my $filename = File::Basename::basename($path);
+    my (undef, $dir, $filename) = File::Spec->splitpath($path);
 
     my $temp;
 
     if ($args{unsaved})
     {
-        my $text = PLS::Parser::Document::text_from_uri($uri);
+        my $text = $args{text};
+        $text = PLS::Parser::Document::text_from_uri($uri) if (ref $text ne 'SCALAR');
 
         if (ref $text eq 'SCALAR')
         {
-            $temp = File::Temp->new(DIR => Cwd::cwd());
+            $temp = File::Temp->new(DIR => $dir, TEMPLATE => '.XXXXXXXX');
             print {$temp} $$text;
             close $temp;
             $path = $temp->filename;
@@ -82,9 +98,8 @@ sub get_compilation_errors
 {
     my ($path) = @_;
 
-
     my @line_lengths;
-    my $pid = open my $fh, '<', $path or return [];
+    open my $fh, '<', $path or return [];
 
     while (my $line = <$fh>)
     {
@@ -92,15 +107,15 @@ sub get_compilation_errors
         $line_lengths[$.] = length $line;
     }
 
-    waitpid $pid, 0;
+    close $fh;
 
     my $perl = PLS::Parser::Pod->get_perl_exe();
     my $inc = PLS::Parser::Pod->get_clean_inc();
     my @inc = map { "-I$_" } @{$inc // []};
 
-    $pid = open3 my $in, my $out, my $err = gensym, $perl, @inc, '-c', $path or return [];
-    close $in;
-    close $out;
+    my $err = gensym;
+    my $pid = eval { open3 my $in, my $out, $err, $perl, @inc, '-c', $path };
+    return [] if (not $pid or length $@);
 
     my @diagnostics;
 
@@ -132,7 +147,7 @@ sub get_compilation_errors
                 source   => 'perl',
               };
         } ## end if (my ($error, $file,...))
-    } ## end while (my $line = <$err>)
+    }
 
     waitpid $pid, 0;
 
@@ -150,10 +165,18 @@ sub get_perlcritic_errors
 
     my @diagnostics;
 
+    # Mapping from perlcritic severity to LSP severity
+    my %severity_map = (
+        5 => 1,
+        4 => 1,
+        3 => 2,
+        2 => 3,
+        1 => 3
+    );
+
     foreach my $violation (@violations)
     {
-        my $severity = 5 - $violation->severity;
-        $severity = 1 unless ($severity);
+        my $severity = $severity_map{$violation->severity};
 
         my $doc = URI->new();
         $doc->scheme('https');
@@ -191,4 +214,15 @@ sub get_perlcritic_errors
     return \@diagnostics;
 } ## end sub get_perlcritic_errors
 
+sub call_diagnostics_function
+{
+    my ($class, $uri, $unsaved) = @_;
+
+    return Future->done() if (ref $PLS::Server::State::CONFIG ne 'HASH');
+
+    my $text;
+    $text = PLS::Parser::Document::text_from_uri($uri) if $unsaved;
+    return $function->call(args => [$uri, $unsaved, $text, $PLS::Server::State::CONFIG, $PLS::Server::State::ROOT_PATH, PLS::Parser::Pod->get_perl_exe()]);
+}
+
 1;
