@@ -5,8 +5,6 @@ use warnings;
 
 use parent 'PLS::Server::Request';
 
-use Coro;
-
 use PLS::Parser::Document;
 use PLS::Server::Request::Diagnostics::PublishDiagnostics;
 
@@ -28,7 +26,7 @@ sub service
     my $text_document = $self->{params}{textDocument};
     PLS::Parser::Document->open_file(%{$text_document});
 
-    $server->{server_requests}->put(PLS::Server::Request::Diagnostics::PublishDiagnostics->new(uri => $self->{params}{textDocument}{uri}));
+    $server->send_server_request(PLS::Server::Request::Diagnostics::PublishDiagnostics->call_diagnostics_function($text_document->{uri}, 1));
 
     return;
 } ## end sub service
